@@ -74,6 +74,7 @@ uint32_t ibd_ibnetdisc_flags = IBND_CONFIG_MLX_EPI;
 uint64_t ibd_mkey;
 uint64_t ibd_sakey = 0;
 int show_keys = 0;
+char *ibd_nd_format = NULL;
 
 static const char *prog_name;
 static const char *prog_args;
@@ -81,7 +82,7 @@ static const char **prog_examples;
 static struct option *long_opts = NULL;
 static const struct ibdiag_opt *opts_map[256];
 
-const static char *get_build_version(void)
+static const char *get_build_version(void)
 {
 	return "BUILD VERSION: " IBDIAG_VERSION " Build date: " __DATE__ " "
 	    __TIME__;
@@ -163,6 +164,9 @@ void read_ibdiag_config(const char *file)
 		} else if (strncmp(name, "sa_key",
 				   strlen("sa_key")) == 0) {
 			ibd_sakey = strtoull(val_str, 0, 0);
+		} else if (strncmp(name, "nd_format",
+				   strlen("nd_format")) == 0) {
+			ibd_nd_format = strdup(val_str);
 		}
 	}
 
@@ -254,7 +258,7 @@ static int process_opt(int ch, char *optarg)
 		val = strtol(optarg, &endp, 0);
 		if (errno || (endp && *endp != '\0') || val <= 0 ||
 		    val > INT_MAX)
-			IBERROR("Invalid timeout \"%s\".  Timeout requires a "
+			IBEXIT("Invalid timeout \"%s\".  Timeout requires a "
 				"positive integer value < %d.", optarg, INT_MAX);
 		else {
 			madrpc_set_timeout((int)val);
@@ -265,7 +269,7 @@ static int process_opt(int ch, char *optarg)
 		/* srcport is not required when resolving via IB_DEST_LID */
 		if (resolve_portid_str(ibd_ca, ibd_ca_port, &sm_portid, optarg,
 				IB_DEST_LID, 0, NULL) < 0)
-			IBERROR("cannot resolve SM destination port %s",
+			IBEXIT("cannot resolve SM destination port %s",
 				optarg);
 		ibd_sm_id = &sm_portid;
 		break;
@@ -279,7 +283,7 @@ static int process_opt(int ch, char *optarg)
 			errno = 0;
 			ibd_mkey = strtoull(getpass("M_Key: "), &endp, 0);
 			if (errno || *endp != '\0') {
-				IBERROR("Bad M_Key");
+				IBEXIT("Bad M_Key");
 			}
                 }
                 break;
@@ -409,7 +413,7 @@ int ibdiag_process_opts(int argc, char *const argv[], void *cxt,
 	return 0;
 }
 
-void iberror(const char *fn, char *msg, ...)
+void ibexit(const char *fn, char *msg, ...)
 {
 	char buf[512];
 	va_list va;
@@ -496,12 +500,39 @@ conv_cnt_human_readable(uint64_t val64, float *val, int data)
 	return ("");
 }
 
+int is_port_info_extended_supported(ib_portid_t * dest, int port,
+				    struct ibmad_port *srcport)
+{
+	uint8_t data[IB_SMP_DATA_SIZE] = { 0 };
+	uint32_t cap_mask;
+	uint16_t cap_mask2;
+
+	if (!smp_query_via(data, dest, IB_ATTR_PORT_INFO, port, 0, srcport))
+		IBEXIT("port info query failed");
+
+	mad_decode_field(data, IB_PORT_CAPMASK_F, &cap_mask);
+	if (cap_mask & CL_NTOH32(IB_PORT_CAP_HAS_CAP_MASK2)) {
+		mad_decode_field(data, IB_PORT_CAPMASK2_F, &cap_mask2);
+		if (!(cap_mask2 &
+		      CL_NTOH16(IB_PORT_CAP2_IS_PORT_INFO_EXT_SUPPORTED))) {
+			IBWARN("port info capability mask2 = 0x%x doesn't"
+			       " indicate PortInfoExtended support", cap_mask2);
+			return 0;
+		}
+	} else {
+		IBWARN("port info capability mask2 not supported");
+		return 0;
+	}
+
+	return 1;
+}
+
 int is_mlnx_ext_port_info_supported(uint32_t devid)
 {
 	if (ibd_ibnetdisc_flags & IBND_CONFIG_MLX_EPI) {
-		if (devid == 0xc738)
+		if ((devid >= 0xc738 && devid <= 0xc73b) || devid == 0xcb20)
 			return 1;
-		if (devid >= 0x1003 && devid <= 0x1011)
+		if (devid >= 0x1003 && devid <= 0x1016)
 			return 1;
 	}
 	return 0;
@@ -686,171 +717,6 @@ int resolve_portid_str(char *ca_name, uint8_t ca_port, ib_portid_t * portid,
 	return -1;
 }
 
-/* define a common SA query structure
- * This is by no means optimal but it moves the saquery functionality out of
- * the saquery tool and provides it to other utilities.
- */
-bind_handle_t sa_get_bind_handle(void)
-{
-	bind_handle_t handle;
-	handle = calloc(1, sizeof(*handle));
-	if (!handle)
-		IBPANIC("calloc failed");
-
-	resolve_sm_portid(ibd_ca, ibd_ca_port, &handle->dport);
-	if (!handle->dport.lid) {
-		IBWARN("No SM/SA found on port %s:%d",
-			ibd_ca ? "" : ibd_ca,
-			ibd_ca_port);
-		free(handle);
-		return (NULL);
-	}
-
-	handle->dport.qp = 1;
-	if (!handle->dport.qkey)
-		handle->dport.qkey = IB_DEFAULT_QP1_QKEY;
-
-	handle->fd = umad_open_port(ibd_ca, ibd_ca_port);
-	handle->agent = umad_register(handle->fd, IB_SA_CLASS, 2, 1, NULL);
-
-	return handle;
-}
-
-void sa_free_bind_handle(bind_handle_t h)
-{
-	umad_unregister(h->fd, h->agent);
-	umad_close_port(h->fd);
-	free(h);
-}
-
-int sa_query(bind_handle_t h, uint8_t method,
-		    uint16_t attr, uint32_t mod, uint64_t comp_mask,
-		    uint64_t sm_key, void *data, size_t datasz,
-		    struct sa_query_result *result)
-{
-	ib_rpc_t rpc;
-	void *umad, *mad;
-	int ret, offset, len = 256;
-
-	memset(&rpc, 0, sizeof(rpc));
-	rpc.mgtclass = IB_SA_CLASS;
-	rpc.method = method;
-	rpc.attr.id = attr;
-	rpc.attr.mod = mod;
-	rpc.mask = comp_mask;
-	rpc.datasz = datasz;
-	rpc.dataoffs = IB_SA_DATA_OFFS;
-
-	umad = calloc(1, len + umad_size());
-	if (!umad)
-		IBPANIC("cannot alloc mem for umad: %s\n", strerror(errno));
-
-	mad_build_pkt(umad, &rpc, &h->dport, NULL, data);
-
-	mad_set_field64(umad_get_mad(umad), 0, IB_SA_MKEY_F, sm_key);
-
-	if (ibdebug > 1)
-		xdump(stdout, "SA Request:\n", umad_get_mad(umad), len);
-
-	ret = umad_send(h->fd, h->agent, umad, len, ibd_timeout, 0);
-	if (ret < 0) {
-		IBWARN("umad_send failed: attr %u: %s\n",
-			attr, strerror(errno));
-		free(umad);
-		return (-ret);
-	}
-
-recv_mad:
-	ret = umad_recv(h->fd, umad, &len, ibd_timeout);
-	if (ret < 0) {
-		if (errno == ENOSPC) {
-			umad = realloc(umad, umad_size() + len);
-			goto recv_mad;
-		}
-		IBWARN("umad_recv failed: attr 0x%x: %s\n", attr,
-			strerror(errno));
-		free(umad);
-		return (-ret);
-	}
-
-	if ((ret = umad_status(umad)))
-		return ret;
-
-	mad = umad_get_mad(umad);
-
-	if (ibdebug > 1)
-		xdump(stdout, "SA Response:\n", mad, len);
-
-	method = (uint8_t) mad_get_field(mad, 0, IB_MAD_METHOD_F);
-	offset = mad_get_field(mad, 0, IB_SA_ATTROFFS_F);
-	result->status = mad_get_field(mad, 0, IB_MAD_STATUS_F);
-	result->p_result_madw = mad;
-	if (result->status != IB_SA_MAD_STATUS_SUCCESS)
-		result->result_cnt = 0;
-	else if (method != IB_MAD_METHOD_GET_TABLE)
-		result->result_cnt = 1;
-	else if (!offset)
-		result->result_cnt = 0;
-	else
-		result->result_cnt = (len - IB_SA_DATA_OFFS) / (offset << 3);
-
-	return 0;
-}
-
-void sa_free_result_mad(struct sa_query_result *result)
-{
-	if (result->p_result_madw) {
-		free((uint8_t *) result->p_result_madw - umad_size());
-		result->p_result_madw = NULL;
-	}
-}
-
-void *sa_get_query_rec(void *mad, unsigned i)
-{
-	int offset = mad_get_field(mad, 0, IB_SA_ATTROFFS_F);
-	return (uint8_t *) mad + IB_SA_DATA_OFFS + i * (offset << 3);
-}
-
-static const char *ib_sa_error_str[] = {
-	"SA_NO_ERROR",
-	"SA_ERR_NO_RESOURCES",
-	"SA_ERR_REQ_INVALID",
-	"SA_ERR_NO_RECORDS",
-	"SA_ERR_TOO_MANY_RECORDS",
-	"SA_ERR_REQ_INVALID_GID",
-	"SA_ERR_REQ_INSUFFICIENT_COMPONENTS",
-	"SA_ERR_REQ_DENIED",
-	"SA_ERR_STATUS_PRIO_SUGGESTED",
-	"SA_ERR_UNKNOWN"
-};
-
-#define ARR_SIZE(a) (sizeof(a)/sizeof((a)[0]))
-#define SA_ERR_UNKNOWN (ARR_SIZE(ib_sa_error_str) - 1)
-
-static inline const char *ib_sa_err_str(IN uint8_t status)
-{
-	if (status > SA_ERR_UNKNOWN)
-		status = SA_ERR_UNKNOWN;
-	return (ib_sa_error_str[status]);
-}
-
-void sa_report_err(int status)
-{
-	int st = status & 0xff;
-	char sm_err_str[64] = { 0 };
-	char sa_err_str[64] = { 0 };
-
-	if (st)
-		sprintf(sm_err_str, " SM(%s)", ib_get_err_str(st));
-
-	st = status >> 8;
-	if (st)
-		sprintf(sa_err_str, " SA(%s)", ib_sa_err_str((uint8_t) st));
-
-	fprintf(stderr, "ERROR: Query result returned 0x%04x, %s%s\n",
-		status, sm_err_str, sa_err_str);
-}
-
 static unsigned int get_max(unsigned int num)
 {
 	unsigned r = 0;		// r will be lg(num)
@@ -1016,3 +882,13 @@ void dump_portinfo(void *pi, int pisize, int tabs)
 		printf("%s\n", buf);
 	}
 }
+
+op_fn_t *match_op(const match_rec_t match_tbl[], char *name)
+{
+	const match_rec_t *r;
+	for (r = match_tbl; r->name; r++)
+		if (!strcasecmp(r->name, name) ||
+		    (r->alias && !strcasecmp(r->alias, name)))
+			return r->fn;
+	return NULL;
+}
