@@ -94,6 +94,7 @@ struct perf_count perf_count =
 struct perf_count_ext perf_count_ext = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
 #define ALL_PORTS 0xFF
+#define MAX_PORTS 255
 
 /* Notes: IB semantics is to cap counters if count has exceeded limits.
  * Therefore we must check for overflows and cap the counters if necessary.
@@ -303,7 +304,7 @@ static void dump_perfcounters(int extended, int timeout, uint16_t cap_mask,
 		memset(pc, 0, sizeof(pc));
 		if (!pma_query_via(pc, portid, port, timeout,
 				   IB_GSI_PORT_COUNTERS, srcport))
-			IBERROR("perfquery");
+			IBEXIT("perfquery");
 		if (!(cap_mask & IB_PM_PC_XMIT_WAIT_SUP)) {
 			/* if PortCounters:PortXmitWait not supported clear this counter */
 			VERBOSE("PortXmitWait not indicated"
@@ -326,13 +327,13 @@ static void dump_perfcounters(int extended, int timeout, uint16_t cap_mask,
 		if (!(cap_mask & IB_PM_EXT_WIDTH_SUPPORTED) &&
 		    !(cap_mask & IB_PM_EXT_WIDTH_NOIETF_SUP))
 			IBWARN
-			    ("PerfMgt ClassPortInfo 0x%x; No extended counter support indicated\n",
+			    ("PerfMgt ClassPortInfo CapMask 0x%02X; No extended counter support indicated\n",
 			     ntohs(cap_mask));
 
 		memset(pc, 0, sizeof(pc));
 		if (!pma_query_via(pc, portid, port, timeout,
 				   IB_GSI_PORT_COUNTERS_EXT, srcport))
-			IBERROR("perfextquery");
+			IBEXIT("perfextquery");
 		if (aggregate)
 			aggregate_perfcounters_ext(cap_mask);
 		else
@@ -359,11 +360,11 @@ static void reset_counters(int extended, int timeout, int mask,
 	if (extended != 1) {
 		if (!performance_reset_via(pc, portid, port, mask, timeout,
 					   IB_GSI_PORT_COUNTERS, srcport))
-			IBERROR("perf reset");
+			IBEXIT("perf reset");
 	} else {
 		if (!performance_reset_via(pc, portid, port, mask, timeout,
 					   IB_GSI_PORT_COUNTERS_EXT, srcport))
-			IBERROR("perf ext reset");
+			IBEXIT("perf ext reset");
 	}
 }
 
@@ -371,6 +372,8 @@ static int reset, reset_only, all_ports, loop_ports, port, extended, xmt_sl,
     rcv_sl, xmt_disc, rcv_err, extended_speeds, smpl_ctl, oprcvcounters, flowctlcounters,
     vloppackets, vlopdata, vlxmitflowctlerrors, vlxmitcounters, swportvlcong,
     rcvcc, slrcvfecn, slrcvbecn, xmitcc, vlxmittimecc;
+static int ports[MAX_PORTS];
+static int ports_count;
 
 static void common_func(ib_portid_t * portid, int port_num, int mask,
 			unsigned query, unsigned reset,
@@ -383,7 +386,7 @@ static void common_func(ib_portid_t * portid, int port_num, int mask,
 		memset(pc, 0, sizeof(pc));
 		if (!pma_query_via(pc, portid, port_num, ibd_timeout, attr,
 				   srcport))
-			IBERROR("cannot query %s", name);
+			IBEXIT("cannot query %s", name);
 
 		dump_func(buf, sizeof(buf), pc, sizeof(pc));
 
@@ -394,7 +397,7 @@ static void common_func(ib_portid_t * portid, int port_num, int mask,
 	memset(pc, 0, sizeof(pc));
 	if (reset && !performance_reset_via(pc, portid, port, mask, ibd_timeout,
 					    attr, srcport))
-		IBERROR("cannot reset %s", name);
+		IBEXIT("cannot reset %s", name);
 }
 
 static void xmt_sl_query(ib_portid_t * portid, int port, int mask)
@@ -462,19 +465,55 @@ static uint8_t *ext_speeds_reset_via(void *rcvbuf, ib_portid_t * dest,
 	return mad_rpc(srcport, &rpc, dest, rcvbuf, rcvbuf);
 }
 
-static void extended_speeds_query(ib_portid_t * portid, int port, uint64_t ext_mask)
+static uint8_t is_rsfec_mode_active(ib_portid_t * portid, int port,
+				  uint16_t cap_mask)
+{
+	uint8_t data[IB_SMP_DATA_SIZE] = { 0 };
+	uint32_t fec_mode_active = 0;
+	uint32_t pie_capmask = 0;
+	if (cap_mask & IS_PM_RSFEC_COUNTERS_SUP) {
+		if (!is_port_info_extended_supported(portid, port, srcport)) {
+			IBWARN("Port Info Extended not supported");
+			return 0;
+		}
+
+		if (!smp_query_via(data, portid, IB_ATTR_PORT_INFO_EXT, port, 0,
+				   srcport))
+			IBEXIT("smp query portinfo extended failed");
+
+		mad_decode_field(data, IB_PORT_EXT_CAPMASK_F, &pie_capmask);
+		mad_decode_field(data, IB_PORT_EXT_FEC_MODE_ACTIVE_F,
+				 &fec_mode_active);
+		if((pie_capmask &
+		    CL_NTOH32(IB_PORT_EXT_CAP_IS_FEC_MODE_SUPPORTED)) &&
+		   (CL_NTOH16(IB_PORT_EXT_RS_FEC_MODE_ACTIVE) == (fec_mode_active & 0xffff)))
+			return 1;
+	}
+
+	return 0;
+}
+
+static void extended_speeds_query(ib_portid_t * portid, int port,
+				  uint64_t ext_mask, uint16_t cap_mask)
 {
 	int mask = ext_mask;
 
-	if (!reset_only)
-		common_func(portid, port, mask, 1, 0,
+	if (!reset_only) {
+		if (is_rsfec_mode_active(portid, port, cap_mask))
+			common_func(portid, port, mask, 1, 0,
+				    "PortExtendedSpeedsCounters with RS-FEC Active",
+				    IB_GSI_PORT_EXT_SPEEDS_COUNTERS,
+				    mad_dump_port_ext_speeds_counters_rsfec_active);
+		else
+			common_func(portid, port, mask, 1, 0,
 			    "PortExtendedSpeedsCounters",
 			    IB_GSI_PORT_EXT_SPEEDS_COUNTERS,
 			    mad_dump_port_ext_speeds_counters);
+	}
 
 	if ((reset_only || reset) &&
 	    !ext_speeds_reset_via(pc, portid, port, ext_mask, ibd_timeout, srcport))
-		IBERROR("cannot reset PortExtendedSpeedsCounters");
+		IBEXIT("cannot reset PortExtendedSpeedsCounters");
 }
 
 static void oprcvcounters_query(ib_portid_t * portid, int port, int mask)
@@ -563,12 +602,12 @@ static void vlxmittimecc_query(ib_portid_t * portid, int port, int mask)
 
 void dump_portsamples_control(ib_portid_t * portid, int port)
 {
-	char buf[1024];
+	char buf[1280];
 
 	memset(pc, 0, sizeof(pc));
 	if (!pma_query_via(pc, portid, port, ibd_timeout,
 			   IB_GSI_PORT_SAMPLES_CONTROL, srcport))
-		IBERROR("sampctlquery");
+		IBEXIT("sampctlquery");
 
 	mad_dump_portsamples_control(buf, sizeof buf, pc, sizeof pc);
 	printf("# PortSamplesControl: %s port %d\n%s", portid2str(portid),
@@ -656,7 +695,7 @@ static int process_opt(void *context, int ch, char *optarg)
 
 int main(int argc, char **argv)
 {
-	int mgmt_classes[2] = { IB_SA_CLASS, IB_PERFORMANCE_CLASS };
+	int mgmt_classes[3] = { IB_SMI_CLASS, IB_SA_CLASS, IB_PERFORMANCE_CLASS };
 	ib_portid_t portid = { 0 };
 	int mask = 0xffff;
 	uint64_t ext_mask = 0xffffffffffffffffULL;
@@ -666,6 +705,7 @@ int main(int argc, char **argv)
 	uint8_t data[IB_SMP_DATA_SIZE] = { 0 };
 	int start_port = 1;
 	int enhancedport0;
+	char *tmpstr;
 	int i;
 
 	const struct ibdiag_opt opts[] = {
@@ -694,7 +734,7 @@ int main(int argc, char **argv)
 		{"Reset_only", 'R', 0, NULL, "only reset counters"},
 		{0}
 	};
-	char usage_args[] = " [<lid|guid> [[port] [reset_mask]]]";
+	char usage_args[] = " [<lid|guid> [[port(s)] [reset_mask]]]";
 	const char *usage_examples[] = {
 		"\t\t# read local port's performance counters",
 		"32 1\t\t# read performance counters from lid 32, port 1",
@@ -707,6 +747,10 @@ int main(int argc, char **argv)
 		"-R -a 32\t# reset performance counters of all ports",
 		"-R 32 2 0x0fff\t# reset only error counters of port 2",
 		"-R 32 2 0xf000\t# reset only non-error counters of port 2",
+		"-a 32 1-10\t# read performance counters from lid 32, port 1-10, aggregate output",
+		"-l 32 1-10\t# read performance counters from lid 32, port 1-10, output each port",
+		"-a 32 1,4,8\t# read performance counters from lid 32, port 1, 4, and 8, aggregate output",
+		"-l 32 1,4,8\t# read performance counters from lid 32, port 1, 4, and 8, output each port",
 		NULL,
 	};
 
@@ -716,39 +760,66 @@ int main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	if (argc > 1)
-		port = strtoul(argv[1], 0, 0);
+	if (argc > 1) {
+		if (strchr(argv[1], ',')) {
+			tmpstr = strtok(argv[1], ",");
+			while (tmpstr) {
+				ports[ports_count++] = strtoul(tmpstr, 0, 0);
+				tmpstr = strtok(NULL, ",");
+			}
+			port = ports[0];
+		}
+		else if ((tmpstr = strchr(argv[1], '-'))) {
+			int pmin, pmax;
+
+			*tmpstr = '\0';
+			tmpstr++;
+
+			pmin = strtoul(argv[1], 0, 0);
+			pmax = strtoul(tmpstr, 0, 0);
+
+			if (pmin >= pmax)
+				IBEXIT("max port must be greater than min port in range");
+
+			while (pmin <= pmax)
+				ports[ports_count++] = pmin++;
+
+			port = ports[0];
+		}
+		else
+			port = strtoul(argv[1], 0, 0);
+	}
 	if (argc > 2) {
 		ext_mask = strtoull(argv[2], 0, 0);
 		mask = ext_mask;
 	}
 
-	srcport = mad_rpc_open_port(ibd_ca, ibd_ca_port, mgmt_classes, 2);
+	srcport = mad_rpc_open_port(ibd_ca, ibd_ca_port, mgmt_classes, 3);
 	if (!srcport)
-		IBERROR("Failed to open '%s' port '%d'", ibd_ca, ibd_ca_port);
+		IBEXIT("Failed to open '%s' port '%d'", ibd_ca, ibd_ca_port);
 
 	smp_mkey_set(srcport, ibd_mkey);
 
 	if (argc) {
 		if (resolve_portid_str(ibd_ca, ibd_ca_port, &portid, argv[0],
 				       ibd_dest_type, ibd_sm_id, srcport) < 0)
-			IBERROR("can't resolve destination port %s", argv[0]);
+			IBEXIT("can't resolve destination port %s", argv[0]);
 	} else {
 		if (resolve_self(ibd_ca, ibd_ca_port, &portid, &port, 0) < 0)
-			IBERROR("can't resolve self port %s", argv[0]);
+			IBEXIT("can't resolve self port %s", argv[0]);
 	}
 
 	/* PerfMgt ClassPortInfo is a required attribute */
 	memset(pc, 0, sizeof(pc));
 	if (!pma_query_via(pc, &portid, port, ibd_timeout, CLASS_PORT_INFO,
 			   srcport))
-		IBERROR("classportinfo query");
+		IBEXIT("classportinfo query");
 	/* ClassPortInfo should be supported as part of libibmad */
 	memcpy(&cap_mask, pc + 2, sizeof(cap_mask));	/* CapabilityMask */
 	if (!(cap_mask & IB_PM_ALL_PORT_SELECT)) {	/* bit 8 is AllPortSelect */
 		if (!all_ports && port == ALL_PORTS)
-			IBERROR("AllPortSelect not supported");
-		if (all_ports)
+			IBEXIT("AllPortSelect not supported");
+		if (all_ports && port == ALL_PORTS)
 			all_ports_loop = 1;
 	}
 
@@ -773,7 +844,7 @@ int main(int argc, char **argv)
 	}
 
 	if (extended_speeds) {
-		extended_speeds_query(&portid, port, ext_mask);
+		extended_speeds_query(&portid, port, ext_mask, cap_mask);
 		goto done;
 	}
 
@@ -844,18 +915,18 @@ int main(int argc, char **argv)
 
 
 	if (all_ports_loop || (loop_ports && (all_ports || port == ALL_PORTS))) {
-		if (smp_query_via(data, &portid, IB_ATTR_NODE_INFO, 0, 0,
-				  srcport) < 0)
-			IBERROR("smp query nodeinfo failed");
+		if (!smp_query_via(data, &portid, IB_ATTR_NODE_INFO, 0, 0,
+				   srcport))
+			IBEXIT("smp query nodeinfo failed");
 		node_type = mad_get_field(data, 0, IB_NODE_TYPE_F);
 		mad_decode_field(data, IB_NODE_NPORTS_F, &num_ports);
 		if (!num_ports)
-			IBERROR("smp query nodeinfo: num ports invalid");
+			IBEXIT("smp query nodeinfo: num ports invalid");
 
 		if (node_type == IB_NODE_SWITCH) {
-			if (smp_query_via(data, &portid, IB_ATTR_SWITCH_INFO,
-					  0, 0, srcport) < 0)
-				IBERROR("smp query nodeinfo failed");
+			if (!smp_query_via(data, &portid, IB_ATTR_SWITCH_INFO,
+					   0, 0, srcport))
+				IBEXIT("smp query nodeinfo failed");
 			enhancedport0 =
 			    mad_get_field(data, 0, IB_SW_ENHANCED_PORT0_F);
 			if (enhancedport0)
@@ -882,6 +953,19 @@ int main(int argc, char **argv)
 				output_aggregate_perfcounters_ext(&portid,
 								  cap_mask);
 		}
+	} else if (ports_count > 1) {
+		for (i = 0; i < ports_count; i++)
+			dump_perfcounters(extended, ibd_timeout, cap_mask,
+					  &portid, ports[i],
+					  (all_ports && !loop_ports));
+		if (all_ports && !loop_ports) {
+			if (extended != 1)
+				output_aggregate_perfcounters(&portid,
+							      cap_mask);
+			else
+				output_aggregate_perfcounters_ext(&portid,
+								  cap_mask);
+		}
 	} else
 		dump_perfcounters(extended, ibd_timeout, cap_mask, &portid,
 				  port, 0);
@@ -896,6 +980,9 @@ do_reset:
 	if (all_ports_loop || (loop_ports && (all_ports || port == ALL_PORTS))) {
 		for (i = start_port; i <= num_ports; i++)
 			reset_counters(extended, ibd_timeout, mask, &portid, i);
+	} else if (ports_count > 1) {
+		for (i = 0; i < ports_count; i++)
+			reset_counters(extended, ibd_timeout, mask, &portid, ports[i]);
 	} else
 		reset_counters(extended, ibd_timeout, mask, &portid, port);
 
